from random import shuffle

###################################################################################################
##########                         EXAMPLE: CARD AND DECK CLASSES                        ##########
###################################################################################################
##########                       EXERCISE: ADD METHODS TO CARD CLASS                     ##########
###################################################################################################
# Step 1: Create a method numerical_value that returns a number to represent each card’s rank. 
#         The method should take one arguments in addition to self: a bool aces_high.
#         Face cards should have values jack = 11, queen = 12, king = 13, ace = 1 or 14 
#         (depending on aces_high)
# Step 2: Create a method higher_rank that takes two arguments in addition to self (a Card other 
#         and a bool aces_high) and returns True if the self.rank is greater than other.rank
#         You may find it useful to call the numerical_value method
# Step 3: Come up with a few good test cases for each method and test the methods
###################################################################################################

class Card:
    """
    A class used to represent a playing card
    
    Attributes
    ----------
    suit (string): the suit of the card
    rank (string or int): the rank of the card
    
    Methods
    -------
    numerical_value(aces_high)
        Returns the rank as a numerical value (Aces are 14 if aces_high is True, otherwise 1)
    higher_rank(other, aces_high)
        Returns True if this card has a higher rank than other
    """
    def __init__(self, suit, rank):
        """
        Initializes a Card object
        
        Arguments:
            suit (string): the suit of the card
            rank (string or int): the rank of the card
        """ 
        self.suit = suit
        self.rank = rank

    def numerical_value(self, aces_high):
        """
        Returns the numerical value of a playing card's rank
        
        Arguments:
            aces_high (bool): whether aces are high or low
             
        Returns:
            int: the value of the card
        """ 
        if type(self.rank) == int:
            return self.rank
        elif self.rank == "K":
            return 13
        elif self.rank == "Q":
            return 12
        elif self.rank == "J":
            return 11
        elif self.rank == "A" and aces_high:
            return 14
        elif self.rank == "A" and not aces_high:
            return 1
        
    def higher_rank(self, other, aces_high):
        """
        Returns whether the card has a higher rank than other card
        
        Arguments:
            other (Card): the card to compare to
            aces_high (bool): whether aces are high or low
             
        Returns:
            bool: 
        """ 
        return self.numerical_value(aces_high) > other.numerical_value(aces_high)
    
    def __str__(self):
        return "Card(" + self.suit + ", " + str(self.rank) + ")"
    
    def __eq__(self, other):
        return self.suit == other.suit and self.rank == other.rank


print("Should be True:", Card("Spades", "J").higher_rank(Card("Spades", 10), True))
print("Should be True:", Card("Spades", "A").higher_rank(Card("Spades", "K"), True))
print("Should be False:", Card("Spades", "A").higher_rank(Card("Spades", "K"), False))

class Deck:
    """
    A class used to represent a playing card
    
    Attributes
    ----------
    cards (List[card]): the cards in the deck
    
    Methods
    -------
    draw()
        Removes a card from the deck and returns it
    """
    def __init__(self):
        """
        Initializes a Deck object
        
        Arguments: None
        """ 
        self.cards = []
        for rank in [2, 3, 4, 5, 6, 7, 8, 9, 10, "J", "Q", "K", "A"]:
            for suit in ["Spades", "Clubs", "Hearts", "Diamonds"]:
                self.cards.append(Card(suit, rank))
        shuffle(self.cards)

    def draw(self):
        """
        Returns the card at index 0 from the deck and removes it
        
        Arguments: None

        Modifies:
            self.cards
             
        Returns:
            Card: the card at the top of the deck (index 0)
        """
        return self.cards.pop(0)


a_card = Card("Hearts", "J")
print(a_card.suit)
a_card.suit = "Diamonds"
print(a_card)
print(a_card == Card("Diamonds", "J"))


###################################################################################################
##########                             EXAMPLE: STUDENT CLASS                            ##########
###################################################################################################
##########               EXERCISE: ADD ATTRIBUTES AND METHODS TO CARD CLASS              ##########
###################################################################################################

class Student:
    """
    A class used to represent a student
    
    Attributes
    ----------
    first_name (string): the student's given name
    last_name (string): the student's family name
    last_first (string): whether the last name comes first in the full name
    current_courses (List[string]): courses that the student is currently registered for
    
    Methods
    -------
    full_name()
        Returns the student's full name
    current_credits()
        Returns the number of credits that the student is registered for
    drop_course()
        Removes a course from the list of current courses
    """
    def __init__(self, first_name, last_name, last_first, current_courses):
        """
        Initializes a Student object
        
        Arguments:
            first_name (string): the student's given name
            last_name (string): the student's family name
            last_first (string): whether the last name comes first in the full name
            current_courses (List[string]): courses that the student is currently registered for
        """
        self.first_name = first_name
        self.last_name = last_name
        # last_first: whether the full name should be returned with the family name first
        self.last_first = last_first
        self.current_courses = current_courses

    def full_name(self):
        """
        Returns the student's full name
        
        Arguments: None
             
        Returns:
            string: the full name
        """
        if self.last_first:
            return self.last_name + " " + self.first_name
        else:
            return self.first_name + " " + self.last_name

    def current_credits(self):
        """
        Returns the number of credits that a student is enrolled in
        (assumes each course is one credit)

        Arguments: None

        Returns:
            int: number of courses
        """
        return len(self.current_courses)
    
    def drop_course(self, course):
        """
        Drop a course from the courses list

        Args:
            course (string): the course to drop

        Modifies:
            self.current_courses

        Returns: None
        """
        self.current_courses.pop(self.current_courses.index(course))


s = Student("Laura", "Biester", False, ["CS 145", "CS 457", "ART 155", "CS 201"])
print(s.current_courses)
s.drop_course("ART 155")
print(s.current_courses)